
__doc__ = """ZCalendar Product"""
__version__ = '0.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
import Acquisition
import AccessControl.Role

from time import time, gmtime, localtime, mktime, asctime, ctime


_days_sunday = ['Su','Mo','Tu','We','Th','Fr','Sa']
_days_monday = ['Mo','Tu','We','Th','Fr','Sa','Su']
_months = ['January','February','March','April','May','June','July','August','September','October','November','December']
_mdays = [31,28,31,30,31,30,31,31,30,31,30,31]

# ---------------------------------------------------------------------------
manage_addCalendarForm = HTMLFile( 'calAdd', globals())

# ---------------------------------------------------------------------------
def manage_addCalendar( self, id, title='', week_starts='monday',REQUEST=None):
    """ add a calendar """
    self._setObject(id,Calendar(id,title, week_starts))
    if REQUEST is not None:
	return self.manage_main( self, REQUEST )
    
# ===========================================================================
class Calendar( OFS.SimpleItem.Item, Persistent,
                Acquisition.Implicit, AccessControl.Role.RoleManager
                ):
    """ ZCalendar Product """
    meta_type = "ZCalendar"
    icon = 'misc_/EconoAccount/calendar.gif'
    manage_options = (
      {'label' : 'Edit',     'action': 'manage_main' },
      {'label' : 'View',     'action': '' },
      {'label' : 'Security', 'action': 'manage_access' },
    )
    
    __ac_permissions__ = (
      ('View management screens', ('manage_tabs','manage_main')),
      ('Change permissions',      ('manage_access',)),
      ('Change Calendar',         ('manage_edit',)),
      ('View Calendar',           ('',)),
    )
    
    # ----------------------------------------------------------------
    def __init__( self, id, title='', week_starts='monday'):
	""" init """
	self.id = id
	self.title = title
	self.week_start = week_starts
	self.titlebg = "#87CEFA"
	self.titlefg = "#FFFFFF"
	self.todayfg = "#FF0000"
	self.todaybg = "#FFFFFF"
	self.calfg = "#000000"
	self.calbg = ""
	
    # ----------------------------------------------------------------
    index_html = HTMLFile('index', globals())
    
    # ----------------------------------------------------------------
    manage_main = HTMLFile('calEdit', globals())
    
    # ----------------------------------------------------------------
    def manage_edit( self, title, 
                    titlefg, titlebg,
		    todayfg, todaybg,
		    calfg, calbg,
                    REQUEST=None ):
	""" edit the properties """
	self.title = title
	self.titlefg = titlefg
	self.titlebg = titlebg
	self.todayfg = todayfg
	self.todaybg = todaybg
	self.calfg   = calfg
	self.calbg   = calbg
	if REQUEST is not None:
	    return MessageDialog(
	           title = 'Edited',
		   message = 'Properties for %s changed' % self.id,
		   action = './manage_main',
		   )
		   
    # ----------------------------------------------------------------
    def _isleap( self, year ):
	return year % 4 == 0 and (year % 100 <> 0 or year % 400 == 0 )

    # ----------------------------------------------------------------
    def _weekday( self, year, month, day ):
	secs = mktime((year,month+1,day,0,0,0,0,0,0))
	t = localtime(secs)
	return t[6]

    # ----------------------------------------------------------------
    def _monthrange( self, year, month ):
	day1 = self._weekday( year, month, 1 )
	ndays = _mdays[month] + (month == 1 and self._isleap(year))
	return day1, ndays

    # ------------------------------------------------------------------------
    def _monthcalendar( self, year, month ):
	day1, ndays = self._monthrange( year, month )
	rows = []
	if self.week_starts=='sunday':
            day = 1 - day1
        else:
            day = 2 - day1
	while day <= ndays+1:
	    row = [0,0,0,0,0,0,0]
	    for i in range(7):
		if 1 < day <= ndays+1:
		    row[i] = day - 1
		day = day + 1
	    rows.append(row)
	return rows

    def _translateMonth(self, name):
        """ use the EconoI18N class in the EconoAccount to translate
            months and days """
        return self.getL()[name.upper()]

    def _translateDay(self, name):
        """ use the EconoI18N class in the EconoAccount to translate
            months and days """
        fullname = {'Mo':'MONDAY',      'Tu':'TUESDAY','We':'WEDNESDAY',
                    'Th':'THURSDAY',    'Fr':'FRIDAY',
                    'Sa':'SATURDAY',    'Su':'SUNDAY'}
        return self.getL()[fullname[name]]

    def _weekdays(self):
        " "
        if self.week_starts=='sunday':
            return _days_sunday
        else:
            return _days_monday
    # ------------------------------------------------------------------------
    def show_calendar( self, year=None, month=None, url=None):
	""" return the calendar HTML """

        if month==12:
            monthBack = 11
            monthForward = 1
        elif month==1:
            monthBack = 12
            monthForward = 2
        else:
            monthBack = month -1
            monthForward = month +1

        if url is None:
            url = self.ActionURL(self.REQUEST.URL)
        p='?'
        if p in url:
            p='&'
                
	monthButtonLeft = """<img width="10" height="11"
                            src="/misc_/EconoAccount/littlearrow_month_left.gif"
                            alt="One month back" border="0" />"""
	monthButtonLeft = '<a href="%s%sCalendarMonth=%s">%s</a>'%(url, p,
                                                                  monthBack+1,
                                                                  monthButtonLeft)
	monthButtonRight = """<img width="10" height="11"
                            src="/misc_/EconoAccount/littlearrow_month_right.gif"
                            alt="One month forward" border="0" />"""
	monthButtonRight = '<a href="%s%sCalendarMonth=%s">%s</a>'%(url, p,
                                                                  monthForward+1,
                                                                  monthButtonRight)
	
	t = localtime(time())
	if year == None:
	    year = t[0]
	if month == None:
	    month = t[1]-1
	cal = self._monthcalendar( year, month )
	# shortcut
        JS = self._JSHyperlinkDate
        if self.calbg.strip()=='':
            bgcolor_tag=''
        else:
            bgcolor_tag='bgcolor="%s"'%self.calbg
        if self.calfg.strip()=='':
            fgcolor_tag=''
        else:
            fgcolor_tag='fgcolor="%s"'%self.calfg
        s = ''
        s = s + self._JavaScript()
	s = s + '<table border=0 cellspacing=1 cellpadding=2 bgcolor="%s" %s %s>\n'%\
            (self.titlebg, bgcolor_tag, fgcolor_tag)
	s = s + '<tr><td>'
	s = s + monthButtonLeft
	s = s + '</td><td colspan=5 align=center>'
	s = s + '<font color="%s">' % self.titlefg
	s = s + '<b>%s %d</b></font>' % (self._translateMonth(_months[month]),
                                         year)
	s = s + '</td><td align="right">'
	s = s + monthButtonRight
	s = s + '</td></tr>\n<tr>'
	for day in self._weekdays():
	    s = s + '<td class="weekdays">%s</td>\n' % self._translateDay(day)
	s = s + '</tr>\n'
	for week in cal:
	    s = s + '<tr bgcolor="#FFFFFF">'
	    for day in week:
		s = s + '<td width="14%" align=center '
		if day:
		    if day == t[2] and month == t[1]-1 and year == t[0]:
			s = s +  'bgcolor="%s">' % self.todaybg
			s = s + '<font color="%s">%s</font>' % \
                            (self.todayfg,JS(str(day), year, month+1,"calendar_daylink_today"))
		    else:
			s = s + '>'
			s = s + '%s' % JS(str(day),year,month+1)
		else:
		    s = s + '>&nbsp;'
		s = s + '</td>'
	    s = s + '</tr>\n'
	s = s + '</table>\n'
	return s

    def _JavaScript(self):
        """ Return javascript declaration """
        return """
        <script language="javascript" type="text/javascript">
        function CID(dstr) {
        window.status='Change input date to ' + dstr;
        return true;
        }
        function noCID(dstr) {
        window.status='';
        return true;
        }
        </script>"""
    def _JSHyperlinkDate(self, day, year, month, css_class="calendar_daylink"):
        """ return the date wrapped in a HTML hyperlink """
        dateString = '%s-%s-%s'% (year, self._zfill(month), self._zfill(day))
        return """<a onClick="changeDatumForm('%s')"
                     onMouseOver="CID('%s')"
                     onMouseOut="noCID()"                     
                     class="%s">%s</a>"""%\
                     (dateString, dateString, css_class, day)
    def _zfill(self, i):
        """ custom simple zfill """
        s=str(i)
        if len(s)==1:
            return "0%s"%s
        else:
            return s
        
